// DirectDrawSurface.cpp: CDirectDrawSurface NX̃Cve[V
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DirectDrawSurface.h"

//////////////////////////////////////////////////////////////////////
// \z/
//////////////////////////////////////////////////////////////////////

CDirectDrawSurface::CDirectDrawSurface()
{
	m_pSurface = NULL;
	m_Width = 0;
	m_Height = 0;
	m_bLocked = false;

	m_Desc.dwSize = sizeof(m_Desc);
}

CDirectDrawSurface::~CDirectDrawSurface()
{
	Release();
}

void CDirectDrawSurface::Initialize(IDirectDrawSurface4** pSuf)
{
	m_pSurface = *pSuf;
	m_pSurface->AddRef();

	m_pSurface->GetSurfaceDesc(&m_Desc);

	m_Width = m_Desc.dwWidth;
	m_Height = m_Desc.dwHeight;
	m_BPP = m_Desc.ddpfPixelFormat.dwRGBBitCount;

		// ꂼ̃}XNɂāArbg𒲂ׁALSBiŉʃrbgj
		// 牽rbgVtgĂ邩𒲂ׂ
		m_RMask = m_Desc.ddpfPixelFormat.dwRBitMask;
		m_GMask = m_Desc.ddpfPixelFormat.dwGBitMask;
		m_BMask = m_Desc.ddpfPixelFormat.dwBBitMask;

		m_RShift = m_RBits = 0;
		m_GShift = m_GBits = 0;
		m_BShift = m_BBits = 0;

		DWORD d = m_RMask;
		while((d&1)==0){	d >>= 1;	m_RShift++;	}
		while(d&1){			d >>= 1;	m_RBits++;	}
		d = m_GMask;
		while((d&1)==0){	d >>= 1;	m_GShift++;	}
		while(d&1){			d >>= 1;	m_GBits++;	}
		d = m_BMask;
		while((d&1)==0){	d >>= 1;	m_BShift++;	}
		while(d&1){			d >>= 1;	m_BBits++;	}
}

void CDirectDrawSurface::Release()
{
	if( m_pSurface!=NULL )
	{
		m_pSurface->Release();
		m_pSurface = NULL;
	}
}

HRESULT CDirectDrawSurface::Blt(RECT * rcTo, CDirectDrawSurface * pSuf, RECT * rcFrom, DWORD flags, DDBLTFX * fx)
{
	if( m_pSurface==NULL )
		return DDERR_INVALIDOBJECT;

	if( pSuf==NULL )
		return m_pSurface->Blt( rcTo , NULL , rcFrom , flags , fx );

	return m_pSurface->Blt( rcTo , pSuf->m_pSurface , rcFrom , flags , fx );
}

void CDirectDrawSurface::SetColorKey(COLORREF cl)
{
	if( m_pSurface==NULL )
		return;

	DWORD dcl;
	COLORREF rgbT;
	HDC hdc;
	HRESULT hr;

	if(cl!=CLR_INVALID && GetDC(&hdc)==DD_OK )
	{
		rgbT = GetPixel(hdc, 0, 0);
		SetPixel(hdc, 0, 0, cl);
		ReleaseDC(hdc);
	}

    DDSURFACEDESC2 ddsd;
    ddsd.dwSize = sizeof(ddsd);
    while ((hr = m_pSurface->Lock(NULL, &ddsd, 0, NULL)) == DDERR_WASSTILLDRAWING);
    if(hr==DD_OK)
    {
		dcl = *(DWORD *)ddsd.lpSurface;
		if(ddsd.ddpfPixelFormat.dwRGBBitCount < 32)
			dcl &= (1 << ddsd.ddpfPixelFormat.dwRGBBitCount)-1;
		m_pSurface->Unlock(NULL);
    }

	if(cl!=CLR_INVALID && GetDC(&hdc)==DD_OK )
	{
		SetPixel(hdc, 0, 0, rgbT);
		ReleaseDC(hdc);
	}

	DDCOLORKEY ddck;
	ddck.dwColorSpaceHighValue = ddck.dwColorSpaceLowValue = dcl;
	hr = m_pSurface->SetColorKey(DDCKEY_SRCBLT, &ddck);
}

HRESULT CDirectDrawSurface::GetDC(HDC* pHDC)
{
	if( m_pSurface==NULL )
		return DDERR_INVALIDOBJECT;

	HRESULT hr;

	retry:

	hr = m_pSurface->GetDC(pHDC);

	switch( hr )
	{
	case DDERR_SURFACELOST:
		m_pSurface->Restore();
		goto retry;
	case DDERR_WASSTILLDRAWING:
		goto retry;
	}

	return hr;
}

void CDirectDrawSurface::ReleaseDC(HDC hdc)
{
	if( m_pSurface==NULL )
		return;

	m_pSurface->ReleaseDC(hdc);
}

bool CDirectDrawSurface::Lock()
{
	if( m_pSurface==NULL )
		return false;

	if( DD_OK == m_pSurface->Lock(NULL,&m_Desc,DDLOCK_SURFACEMEMORYPTR|DDLOCK_WAIT,NULL) )
		m_bLocked=true;

	return true;
}

void CDirectDrawSurface::Unlock()
{
	if( m_bLocked )
		m_pSurface->Unlock(NULL);
}

BYTE* CDirectDrawSurface::GetHeadPtr(int line)
{
	// ( 0,line )ւ̃|C^(1byte)擾
	if( !m_bLocked )
		return NULL;

	BYTE* pBuf = (BYTE*)(m_Desc.lpSurface);
	pBuf += line * m_Desc.lPitch;

	return pBuf;
}

DWORD CDirectDrawSurface::GetOffset()
{
	// 1hbgEɃ|C^𓮂߂̃ItZbg
	switch(m_BPP)
	{
	case 8:
		return 1;
	case 16:
		return 2;
	case 24:
		return 3;
	case 32:
		return 4;
	}
	return 0;
}

DWORD CDirectDrawSurface::RGB2SRF( int r,int g,int b )
{
	return ( (r >> (8 - m_RBits)) << m_RShift
		   | (g >> (8 - m_GBits)) << m_GShift
	 	   | (b >> (8 - m_BBits)) << m_BShift );
}

void CDirectDrawSurface::SRF2RGB( DWORD dw , int* r, int* g, int* b )
{
	*r = ( ( (dw&m_RMask) >> m_RShift) << (8-m_RBits) );
	*g = ( ( (dw&m_GMask) >> m_GShift) << (8-m_GBits) );
	*b = ( ( (dw&m_BMask) >> m_BShift) << (8-m_BBits) );
}

void CDirectDrawSurface::SetPixelDD( BYTE* p , DWORD cl )
{
	switch(m_BPP)
	{
	case 8:
		{
			break;
		}
	case 16:
		{
			WORD* pW = (WORD*)p;
			*pW = (WORD)cl;
			break;
		}
	case 24:
		{
			break;
		}
	case 32:
		{
			DWORD* pW = (DWORD*)p;
			*pW = (DWORD)cl;
			break;
		}
	}
}

void CDirectDrawSurface::GetPixelDD( BYTE* p , DWORD* pD )
{
	switch(m_BPP)
	{
	case 8:
		{
			break;
		}
	case 16:
		{
			WORD* pW = (WORD*)p;
			WORD W = *pW;
			*pD = (DWORD)W;
			break;
		}
	case 24:
		{
			break;
		}
	case 32:
		{
			DWORD* pW = (DWORD*)p;
			DWORD W = *pW;
			*pD = (DWORD)W;
			break;
		}
	}
}
